/*++

Copyright (c) Sydus.

Module:

    Logger.h

Authors:

    Sharvil Nanavati (sharvil) 2005-10-08

--*/

#pragma once

#include "base.h"

#include <string>
#include <cstdio>
#include <set>

class ILogSink;

enum ELogLevel
{
	KLogDebug,
	KLogInfo,
	KLogWarning,
	KLogError,
	KLogDisable    // A special level that does not perform any logging.
};

class Logger : public NonCopyable
{
	public:
		static Logger & Instance();

		void AttachSink(ILogSink & sink);
		void DetachSink(ILogSink & sink);

		void Log(const ELogLevel & level, const_cstring format, ...);

		void SetLogFile(const std::string & filename);
		void SetLogThreshold(const ELogLevel & minimumLevel);
		ELogLevel GetLogThreshold() const;
		bool IsEnabled(const ELogLevel & level) const;

		~Logger();

	private:
		typedef std::set <ILogSink *> Sinks;

		Logger();

		std::string LevelName(const ELogLevel & level) const;

		Sinks iSinks;
		FILE * iLogFile;
		std::string iFilename;
		ELogLevel iCurrentLogThreshold;
};
