/*++

Copyright (c) Sydus.

Module:

    ISocketImplementation.h

Authors:

    Sharvil Nanavati (sharvil) 2006-07-30

--*/

#pragma once

#include "base.h"

class Socket;
class InetAddress;

class ISocketImplementation
{
	public:
		virtual ~ISocketImplementation() {}

		virtual ISocketImplementation * Accept() = 0;
		virtual void Bind(const InetAddress & ip, uint16 port) = 0;
		virtual void Listen() = 0;

		virtual void Connect(const std::string & host, uint16 port) = 0;
		virtual void Close() = 0;

		virtual uint32 Read(void * buffer, uint32 size) const = 0;
		virtual uint32 Write(const void * buffer, uint32 size) const = 0;

		virtual uint32 Read(InetAddress & source, void * buffer, uint32 size) const = 0;
		virtual uint32 Write(const InetAddress & destination, const void * buffer, uint32 size) const = 0;

		virtual bool IsReadable() const = 0;
		virtual bool IsWritable() const = 0;

		virtual const InetAddress & GetRemoteAddress() const = 0;

		virtual void SetReuseAddress(bool reuse) = 0;
		virtual void SetBlocking() = 0;
		virtual void SetNonBlocking() = 0;
		virtual void EnableKeepAlive(int32 idleTime = -1, int32 interval = -1, int32 numProbes = -1) = 0;
		virtual void DisableKeepAlive() = 0;
		virtual void EnableNagle() = 0;
		virtual void DisableNagle() = 0;

	protected:
		friend class Socket;
		virtual int32 GetFd() const = 0;
};
