/*++

Copyright (c) Sydus.

Module:

    TimeLibTime.h

Authors:

    Sharvil Nanavati (sharvil) 2005-10-18

--*/

#pragma once

#include "base.h"

#ifndef PLATFORM_WIN32
#	include <sys/time.h>
#endif

enum EDayOfWeek
{
	KSunday,
	KMonday,
	KTuesday,
	KWednesday,
	KThursday,
	KFriday,
	KSaturday
};

class Time
{
	private:

#ifdef PLATFORM_WIN32
		typedef SYSTEMTIME time_type;
#else
		typedef struct timeval time_type;
#endif

	public:
		Time();
		~Time();

		static void Sleep(uint32 milliseconds);
		static uint32 GetMilliCount();

		Time & Update();

		int32 GetDay() const;
		int32 GetYear() const;
		int32 GetMonth() const;
		int32 GetHour() const;
		int32 GetMinute() const;
		int32 GetSecond() const;
		EDayOfWeek GetDayOfWeek() const;

		int32 GetLocalDay() const;
		int32 GetLocalYear() const;
		int32 GetLocalMonth() const;
		int32 GetLocalHour() const;
		int32 GetLocalMinute() const;
		int32 GetLocalSecond() const;
		EDayOfWeek GetLocalDayOfWeek() const;

		//
		// Warning: these routines are not portable - the definition
		// of epoch is platform-dependent (i.e. Windows vs. UNIX)
		//
		uint64 SecondsSinceUtcEpoch() const;
		uint64 MillisSinceUtcEpoch() const;

		std::string ToUtcString() const;
		std::string ToLocalTzString() const;
		std::string ToHttpString() const;

	private:
		time_type iUtcTime;
		time_type iLocalTime;
};
